@@ -1,19 +1,22 @@
-use super::utils::editor;
+use std::collections::BTreeMap;
+use std::fs::write;
+use std::path::PathBuf;
+use std::process::Command;
+use std::{env, fs};
+
 use anyhow::Result;
 use directories::BaseDirs;
 use log::{debug, LevelFilter};
 use pretty_env_logger::formatted_timed_builder;
 use regex::Regex;
 use serde::Deserialize;
-use std::collections::BTreeMap;
-use std::fs::write;
-use std::path::PathBuf;
-use std::process::Command;
-use std::{env, fs};
 use structopt::StructOpt;
-use strum::{EnumIter, EnumString, EnumVariantNames, IntoEnumIterator, VariantNames};
+use strum::{EnumIter, EnumString, EnumVariantNames, VariantNames};
+use sys_info::hostname;
 use which_crate::which;
 
+use super::utils::editor;
+
 pub static EXAMPLE_CONFIG: &str = include_str!("../config.example.toml");
 
 #[allow(unused_macros)]
@@ -96,6 +99,7 @@ pub enum Step {
     Nix,
     Node,
     Opam,
+    Pacdiff,
     Pacstall,
     Pearl,
     Pipx,
@@ -115,6 +119,7 @@ pub enum Step {
     Sheldon,
     Shell,
     Snap,
+    Spicetify,
     Stack,
     System,
     Tldr,
@@ -195,6 +200,7 @@ pub enum ArchPackageManager {
 pub struct Linux {
     yay_arguments: Option<String>,
     arch_package_manager: Option<ArchPackageManager>,
+    show_arch_news: Option<bool>,
     trizen_arguments: Option<String>,
     dnf_arguments: Option<String>,
     apt_arguments: Option<String>,
@@ -211,6 +217,12 @@ pub struct Composer {
     self_update: Option<bool>,
 }
 
+#[derive(Deserialize, Default, Debug)]
+#[serde(deny_unknown_fields)]
+pub struct Vim {
+    force_plug_update: Option<bool>,
+}
+
 #[derive(Deserialize, Default, Debug)]
 #[serde(deny_unknown_fields)]
 /// Configuration file
@@ -243,6 +255,7 @@ pub struct ConfigFile {
     git: Option<Git>,
     windows: Option<Windows>,
     npm: Option<NPM>,
+    vim: Option<Vim>,
     firmware: Option<Firmware>,
     vagrant: Option<Vagrant>,
     flatpak: Option<Flatpak>,
@@ -380,9 +393,9 @@ pub struct CommandLineArgs {
     #[structopt(short = "k", long = "keep")]
     keep_at_end: bool,
 
-    /// Say yes to package manager's prompt (experimental)
+    /// Say yes to package manager's prompt
     #[structopt(short = "y", long = "yes")]
-    yes: bool,
+    yes: Option<Vec<Step>>,
 
     /// Don't pull the predefined git repos
     #[structopt(long = "disable-predefined-git-repos")]
@@ -497,20 +510,18 @@ impl Config {
     }
 
     fn allowed_steps(opt: &CommandLineArgs, config_file: &ConfigFile) -> Vec<Step> {
-        let mut enabled_steps: Vec<Step> = if !opt.only.is_empty() {
-            opt.only.clone()
-        } else {
-            config_file
-                .only
-                .as_ref()
-                .map_or_else(|| Step::iter().collect(), |v| v.clone())
-        };
+        let mut enabled_steps: Vec<Step> = Vec::new();
+        enabled_steps.extend(&opt.only);
 
-        let disabled_steps: Vec<Step> = if !opt.disable.is_empty() {
-            opt.disable.clone()
-        } else {
-            config_file.disable.as_ref().map_or_else(Vec::new, |v| v.clone())
-        };
+        if let Some(only) = config_file.only.as_ref() {
+            enabled_steps.extend(only)
+        }
+
+        let mut disabled_steps: Vec<Step> = Vec::new();
+        disabled_steps.extend(&opt.disable);
+        if let Some(disabled) = config_file.disable.as_ref() {
+            disabled_steps.extend(disabled);
+        }
 
         enabled_steps.retain(|e| !disabled_steps.contains(e) || opt.only.contains(e));
         enabled_steps
@@ -574,8 +585,20 @@ impl Config {
 
     /// Whether to say yes to package managers
     #[allow(dead_code)]
-    pub fn yes(&self) -> bool {
-        self.config_file.assume_yes.unwrap_or(self.opt.yes)
+    pub fn yes(&self, step: Step) -> bool {
+        if let Some(yes) = self.config_file.assume_yes {
+            return yes;
+        }
+
+        if let Some(yes_list) = &self.opt.yes {
+            if yes_list.is_empty() {
+                return true;
+            }
+
+            return yes_list.contains(&step);
+        }
+
+        false
     }
 
     /// Bash-it branch
@@ -625,6 +648,15 @@ impl Config {
             .unwrap_or(false)
     }
 
+    /// Whether to force plug update in Vim
+    pub fn force_vim_plug_update(&self) -> bool {
+        self.config_file
+            .vim
+            .as_ref()
+            .and_then(|c| c.force_plug_update)
+            .unwrap_or_default()
+    }
+
     /// Whether to send a desktop notification at the beginning of every step
     #[allow(dead_code)]
     pub fn notify_each_step(&self) -> bool {
@@ -641,6 +673,16 @@ impl Config {
             .unwrap_or("")
     }
 
+    /// Show news on Arch Linux
+    #[allow(dead_code)]
+    pub fn show_arch_news(&self) -> bool {
+        self.config_file
+            .linux
+            .as_ref()
+            .and_then(|s| s.show_arch_news)
+            .unwrap_or(true)
+    }
+
     /// Extra yay arguments
     #[allow(dead_code)]
     pub fn arch_package_manager(&self) -> ArchPackageManager {
@@ -797,10 +839,16 @@ impl Config {
     str_value!(linux, emerge_update_flags);
 
     pub fn should_execute_remote(&self, remote: &str) -> bool {
-        self.opt
-            .remote_host_limit
-            .as_ref()
-            .map(|h| h.is_match(remote))
-            .unwrap_or(true)
+        if let Ok(hostname) = hostname() {
+            if remote == hostname {
+                return false;
+            }
+        }
+
+        if let Some(limit) = self.opt.remote_host_limit.as_ref() {
+            return limit.is_match(remote);
+        }
+
+        true
     }
 }
