@@ -138,6 +138,8 @@ fn run() -> Result<()> {
             linux::run_etc_update(sudo.as_ref(), run_type)
         })?;
 
+        runner.execute(Step::Pacdiff, "pacdiff", || linux::run_pacdiff(sudo.as_ref(), run_type))?;
+
         runner.execute(Step::BrewFormula, "Brew", || {
             unix::run_brew_formula(&ctx, unix::BrewVariant::Linux)
         })?;
@@ -290,7 +292,7 @@ fn run() -> Result<()> {
     runner.execute(Step::Choosenim, "choosenim", || generic::run_choosenim(&ctx))?;
     runner.execute(Step::Cargo, "cargo", || generic::run_cargo_update(&ctx))?;
     runner.execute(Step::Flutter, "Flutter", || generic::run_flutter_upgrade(run_type))?;
-    runner.execute(Step::Emacs, "Emacs", || emacs.upgrade(run_type))?;
+    runner.execute(Step::Emacs, "Emacs", || emacs.upgrade(&ctx))?;
     runner.execute(Step::Opam, "opam", || generic::run_opam_update(run_type))?;
     runner.execute(Step::Vcpkg, "vcpkg", || generic::run_vcpkg_update(run_type))?;
     runner.execute(Step::Pipx, "pipx", || generic::run_pipx_update(run_type))?;
@@ -322,6 +324,7 @@ fn run() -> Result<()> {
     })?;
     runner.execute(Step::Micro, "micro", || generic::run_micro(run_type))?;
     runner.execute(Step::Raco, "raco", || generic::run_raco_update(run_type))?;
+    runner.execute(Step::Spicetify, "spicetify", || generic::spicetify_upgrade(&ctx))?;
 
     #[cfg(target_os = "linux")]
     {
@@ -410,10 +413,16 @@ fn run() -> Result<()> {
         print_info("\n(R)eboot\n(S)hell\n(Q)uit");
         loop {
             match get_key() {
-                Ok(Key::Char('s' | 'S')) => run_shell(),
-                Ok(Key::Char('r' | 'R')) => reboot(),
-                Ok(Key::Char('q' | 'Q')) => (),
-                _ => continue,
+                Ok(Key::Char('s')) | Ok(Key::Char('S')) => {
+                    run_shell();
+                }
+                Ok(Key::Char('r')) | Ok(Key::Char('R')) => {
+                    reboot();
+                }
+                Ok(Key::Char('q')) | Ok(Key::Char('Q')) => (),
+                _ => {
+                    continue;
+                }
             }
             break;
         }
