@@ -4,9 +4,10 @@ use crate::executor::{CommandExt, RunType};
 use crate::steps::os::archlinux;
 use crate::terminal::{print_separator, print_warning};
 use crate::utils::{require, require_option, which, PathExt};
+use crate::Step;
 use anyhow::Result;
 use ini::Ini;
-use log::debug;
+use log::{debug, warn};
 use serde::Deserialize;
 use std::path::{Path, PathBuf};
 use std::process::Command;
@@ -25,6 +26,7 @@ struct OsRelease {
 pub enum Distribution {
     Alpine,
     Arch,
+    Bedrock,
     CentOS,
     ClearLinux,
     Fedora,
@@ -46,12 +48,12 @@ impl Distribution {
 
         Ok(match id {
             Some("alpine") => Distribution::Alpine,
-            Some("centos" | "rhel" | "ol") => Distribution::CentOS,
+            Some("centos") | Some("rhel") | Some("ol") => Distribution::CentOS,
             Some("clear-linux-os") => Distribution::ClearLinux,
             Some("fedora") => Distribution::Fedora,
             Some("void") => Distribution::Void,
-            Some("debian" | "pureos") => Distribution::Debian,
-            Some("arch" | "anarchy" | "manjaro-arm" | "garuda" | "artix") => Distribution::Arch,
+            Some("debian") | Some("pureos") => Distribution::Debian,
+            Some("arch") | Some("anarchy") | Some("manjaro-arm") | Some("garuda") | Some("artix") => Distribution::Arch,
             Some("solus") => Distribution::Solus,
             Some("gentoo") => Distribution::Gentoo,
             Some("exherbo") => Distribution::Exherbo,
@@ -79,6 +81,10 @@ impl Distribution {
     }
 
     pub fn detect() -> Result<Self> {
+        if PathBuf::from("/bedrock").exists() {
+            return Ok(Distribution::Bedrock);
+        }
+
         if PathBuf::from(OS_RELEASE_PATH).exists() {
             let os_release = Ini::load_from_file(OS_RELEASE_PATH)?;
 
@@ -104,6 +110,7 @@ impl Distribution {
             Distribution::Exherbo => upgrade_exherbo(ctx),
             Distribution::NixOS => upgrade_nixos(ctx),
             Distribution::KDENeon => upgrade_neon(ctx),
+            Distribution::Bedrock => update_bedrock(ctx),
         }
     }
 
@@ -118,6 +125,31 @@ impl Distribution {
     }
 }
 
+fn update_bedrock(ctx: &ExecutionContext) -> Result<()> {
+    let sudo = require_option(ctx.sudo().as_ref(), String::from("Sudo required"))?;
+
+    ctx.run_type().execute(sudo).args(&["brl", "update"]);
+
+    let output = Command::new("brl").arg("list").output()?;
+    debug!("brl list: {:?} {:?}", output.stdout, output.stderr);
+
+    let parsed_output = String::from_utf8(output.stdout).unwrap();
+    for distribution in parsed_output.trim().split('\n') {
+        debug!("Bedrock distribution {}", distribution);
+        match distribution {
+            "arch" => archlinux::upgrade_arch_linux(ctx)?,
+            "debian" | "ubuntu" => upgrade_debian(ctx)?,
+            "centos" | "fedora" => upgrade_redhat(ctx)?,
+            "bedrock" => upgrade_bedrock_strata(ctx)?,
+            _ => {
+                warn!("Unknown distribution {}", distribution);
+            }
+        }
+    }
+
+    Ok(())
+}
+
 fn is_wsl() -> Result<bool> {
     let output = Command::new("uname").arg("-r").check_output()?;
     debug!("Uname output: {}", output);
@@ -133,11 +165,11 @@ fn upgrade_alpine_linux(ctx: &ExecutionContext) -> Result<()> {
 }
 
 fn upgrade_redhat(ctx: &ExecutionContext) -> Result<()> {
-    let _ = if let Some(ostree) = Path::new("/usr/bin/rpm-ostree").if_exists() {
+    let _ = if let Some(ostree) = Path::new("rpm-ostree").if_exists() {
         if ctx.config().rpm_ostree() {
             let mut command = ctx.run_type().execute(ostree);
             command.arg("upgrade");
-            if ctx.config().yes() {
+            if ctx.config().yes(Step::System) {
                 command.arg("-y");
             }
 
@@ -148,11 +180,7 @@ fn upgrade_redhat(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = &ctx.sudo() {
         let mut command = ctx.run_type().execute(&sudo);
         command
-            .arg(
-                Path::new("/usr/bin/dnf-3")
-                    .if_exists()
-                    .unwrap_or_else(|| Path::new("/usr/bin/yum")),
-            )
+            .arg(which("dnf").unwrap_or_else(|| Path::new("yum").to_path_buf()))
             .arg(if ctx.config().redhat_distro_sync() {
                 "distro-sync"
             } else {
@@ -163,7 +191,7 @@ fn upgrade_redhat(ctx: &ExecutionContext) -> Result<()> {
             command.args(args.split_whitespace());
         }
 
-        if ctx.config().yes() {
+        if ctx.config().yes(Step::System) {
             command.arg("-y");
         }
 
@@ -175,16 +203,23 @@ fn upgrade_redhat(ctx: &ExecutionContext) -> Result<()> {
     Ok(())
 }
 
+fn upgrade_bedrock_strata(ctx: &ExecutionContext) -> Result<()> {
+    if let Some(sudo) = ctx.sudo() {
+        ctx.run_type().execute(&sudo).args(&["brl", "update"]).check_run()?;
+    } else {
+        print_warning("No sudo detected. Skipping system upgrade");
+    }
+
+    Ok(())
+}
+
 fn upgrade_suse(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = ctx.sudo() {
-        ctx.run_type()
-            .execute(&sudo)
-            .args(["/usr/bin/zypper", "refresh"])
-            .check_run()?;
+        ctx.run_type().execute(&sudo).args(&["zypper", "refresh"]).check_run()?;
 
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/zypper", "dist-upgrade"])
+            .args(&["zypper", "dist-upgrade"])
             .check_run()?;
     } else {
         print_warning("No sudo detected. Skipping system upgrade");
@@ -197,12 +232,12 @@ fn upgrade_void(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = ctx.sudo() {
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/xbps-install", "-Su", "xbps"])
+            .args(&["xbps-install", "-Su", "xbps"])
             .check_run()?;
 
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/xbps-install", "-u"])
+            .args(&["xbps-install", "-u"])
             .check_run()?;
     } else {
         print_warning("No sudo detected. Skipping system upgrade");
@@ -216,13 +251,13 @@ fn upgrade_gentoo(ctx: &ExecutionContext) -> Result<()> {
 
     if let Some(sudo) = &ctx.sudo() {
         if let Some(layman) = which("layman") {
-            run_type.execute(&sudo).arg(layman).args(["-s", "ALL"]).check_run()?;
+            run_type.execute(&sudo).arg(layman).args(&["-s", "ALL"]).check_run()?;
         }
 
         println!("Syncing portage");
         run_type
             .execute(&sudo)
-            .args(["/usr/bin/emerge", "--sync"])
+            .args(&["emerge", "--sync"])
             .args(
                 ctx.config()
                     .emerge_sync_flags()
@@ -237,7 +272,7 @@ fn upgrade_gentoo(ctx: &ExecutionContext) -> Result<()> {
 
         run_type
             .execute(&sudo)
-            .arg("/usr/bin/emerge")
+            .arg("emerge")
             .args(
                 ctx.config()
                     .emerge_update_flags()
@@ -254,12 +289,12 @@ fn upgrade_gentoo(ctx: &ExecutionContext) -> Result<()> {
 
 fn upgrade_debian(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = &ctx.sudo() {
-        let apt = which("apt-fast").unwrap_or_else(|| PathBuf::from("/usr/bin/apt-get"));
+        let apt = which("apt-fast").unwrap_or_else(|| PathBuf::from("apt-get"));
         ctx.run_type().execute(&sudo).arg(&apt).arg("update").check_run()?;
 
         let mut command = ctx.run_type().execute(&sudo);
         command.arg(&apt).arg("dist-upgrade");
-        if ctx.config().yes() {
+        if ctx.config().yes(Step::System) {
             command.arg("-y");
         }
         if let Some(args) = ctx.config().apt_arguments() {
@@ -272,7 +307,7 @@ fn upgrade_debian(ctx: &ExecutionContext) -> Result<()> {
 
             let mut command = ctx.run_type().execute(&sudo);
             command.arg(&apt).arg("autoremove");
-            if ctx.config().yes() {
+            if ctx.config().yes(Step::System) {
                 command.arg("-y");
             }
             command.check_run()?;
@@ -286,10 +321,7 @@ fn upgrade_debian(ctx: &ExecutionContext) -> Result<()> {
 
 fn upgrade_solus(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = ctx.sudo() {
-        ctx.run_type()
-            .execute(&sudo)
-            .args(["/usr/bin/eopkg", "upgrade"])
-            .check_run()?;
+        ctx.run_type().execute(&sudo).args(&["eopkg", "upgrade"]).check_run()?;
     } else {
         print_warning("No sudo detected. Skipping system upgrade");
     }
@@ -299,6 +331,11 @@ fn upgrade_solus(ctx: &ExecutionContext) -> Result<()> {
 
 pub fn run_pacstall(ctx: &ExecutionContext) -> Result<()> {
     let pacstall = require("pacstall")?;
+    ctx.run_type()
+        .execute(&pacstall)
+        .arg("-U")
+        .args(ctx.config().yay_arguments().split_whitespace())
+        .check_run()?;
     ctx.run_type()
         .execute(pacstall)
         .arg("-Up")
@@ -308,10 +345,7 @@ pub fn run_pacstall(ctx: &ExecutionContext) -> Result<()> {
 
 fn upgrade_clearlinux(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = &ctx.sudo() {
-        ctx.run_type()
-            .execute(&sudo)
-            .args(["/usr/bin/swupd", "update"])
-            .check_run()?;
+        ctx.run_type().execute(&sudo).args(&["swupd", "update"]).check_run()?;
     } else {
         print_warning("No sudo detected. Skipping system upgrade");
     }
@@ -321,31 +355,28 @@ fn upgrade_clearlinux(ctx: &ExecutionContext) -> Result<()> {
 
 fn upgrade_exherbo(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = ctx.sudo() {
-        ctx.run_type()
-            .execute(&sudo)
-            .args(["/usr/bin/cave", "sync"])
-            .check_run()?;
+        ctx.run_type().execute(&sudo).args(&["cave", "sync"]).check_run()?;
 
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/cave", "resolve", "world", "-c1", "-Cs", "-km", "-Km", "-x"])
+            .args(&["cave", "resolve", "world", "-c1", "-Cs", "-km", "-Km", "-x"])
             .check_run()?;
 
         if ctx.config().cleanup() {
             ctx.run_type()
                 .execute(&sudo)
-                .args(["/usr/bin/cave", "purge", "-x"])
+                .args(&["cave", "purge", "-x"])
                 .check_run()?;
         }
 
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/cave", "fix-linkage", "-x", "--", "-Cs"])
+            .args(&["cave", "fix-linkage", "-x", "--", "-Cs"])
             .check_run()?;
 
         ctx.run_type()
             .execute(&sudo)
-            .args(["/usr/bin/eclectic", "config", "interactive"])
+            .args(&["eclectic", "config", "interactive"])
             .check_run()?;
     } else {
         print_warning("No sudo detected. Skipping system upgrade");
@@ -358,13 +389,13 @@ fn upgrade_nixos(ctx: &ExecutionContext) -> Result<()> {
     if let Some(sudo) = ctx.sudo() {
         ctx.run_type()
             .execute(&sudo)
-            .args(["/run/current-system/sw/bin/nixos-rebuild", "switch", "--upgrade"])
+            .args(&["/run/current-system/sw/bin/nixos-rebuild", "switch", "--upgrade"])
             .check_run()?;
 
         if ctx.config().cleanup() {
             ctx.run_type()
                 .execute(&sudo)
-                .args(["/run/current-system/sw/bin/nix-collect-garbage", "-d"])
+                .args(&["/run/current-system/sw/bin/nix-collect-garbage", "-d"])
                 .check_run()?;
         }
     } else {
@@ -386,7 +417,7 @@ fn upgrade_neon(ctx: &ExecutionContext) -> Result<()> {
         ctx.run_type().execute(&sudo).arg(&pkcon).arg("refresh").check_run()?;
         let mut exe = ctx.run_type().execute(&sudo);
         let cmd = exe.arg(&pkcon).arg("update");
-        if ctx.config().yes() {
+        if ctx.config().yes(Step::System) {
             cmd.arg("-y");
         }
         if ctx.config().cleanup() {
@@ -433,7 +464,7 @@ pub fn run_fwupdmgr(ctx: &ExecutionContext) -> Result<()> {
 
     if ctx.config().firmware_upgrade() {
         updmgr.arg("update");
-        if ctx.config().yes() {
+        if ctx.config().yes(Step::System) {
             updmgr.arg("-y");
         }
     } else {
@@ -445,27 +476,49 @@ pub fn run_fwupdmgr(ctx: &ExecutionContext) -> Result<()> {
 pub fn flatpak_update(ctx: &ExecutionContext) -> Result<()> {
     let flatpak = require("flatpak")?;
     let sudo = require_option(ctx.sudo().as_ref(), String::from("sudo is not installed"))?;
+    let cleanup = ctx.config().cleanup();
     let run_type = ctx.run_type();
     print_separator("Flatpak User Packages");
 
     run_type
         .execute(&flatpak)
-        .args(["update", "--user", "-y"])
+        .args(&["update", "--user", "-y"])
         .check_run()?;
+    if cleanup {
+        run_type
+            .execute(&flatpak)
+            .args(&["uninstall", "--user", "--unused"])
+            .check_run()?;
+    }
 
     print_separator("Flatpak System Packages");
     if ctx.config().flatpak_use_sudo() || std::env::var("SSH_CLIENT").is_ok() {
         run_type
-            .execute(sudo)
-            .arg(flatpak)
-            .args(["update", "--system", "-y"])
-            .check_run()
+            .execute(&sudo)
+            .arg(&flatpak)
+            .args(&["update", "--system", "-y"])
+            .check_run()?;
+        if cleanup {
+            run_type
+                .execute(sudo)
+                .arg(flatpak)
+                .args(&["uninstall", "--system", "--unused"])
+                .check_run()?;
+        }
     } else {
         run_type
             .execute(&flatpak)
-            .args(["update", "--system", "-y"])
-            .check_run()
+            .args(&["update", "--system", "-y"])
+            .check_run()?;
+        if cleanup {
+            run_type
+                .execute(flatpak)
+                .args(&["uninstall", "--system", "--unused"])
+                .check_run()?;
+        }
     }
+
+    Ok(())
 }
 
 pub fn run_snap(sudo: Option<&PathBuf>, run_type: RunType) -> Result<()> {
@@ -498,6 +551,15 @@ pub fn run_etc_update(sudo: Option<&PathBuf>, run_type: RunType) -> Result<()> {
     run_type.execute(sudo).arg(etc_update).check_run()
 }
 
+pub fn run_pacdiff(sudo: Option<&PathBuf>, run_type: RunType) -> Result<()> {
+    let sudo = require_option(sudo, String::from("sudo is not installed"))?;
+    let pacdiff = require("pacdiff")?;
+
+    print_separator("pacdiff");
+
+    run_type.execute(sudo).arg(pacdiff).check_run()
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
